@@ -16,6 +16,7 @@ import random
 from openrazer_daemon.dbus_services.service import DBusService
 import openrazer_daemon.dbus_services.dbus_methods
 from openrazer_daemon.misc import effect_sync
+from openrazer_daemon.misc.battery_notifier import BatteryManager as _BatteryManager
 
 
 # pylint: disable=too-many-instance-attributes
@@ -38,6 +39,7 @@ class RazerDevice(DBusService):
     DEDICATED_MACRO_KEYS = False
     MATRIX_DIMS = None
     POLL_RATES = None
+    DPI_MAX = None
 
     WAVE_DIRS = (1, 2)
 
@@ -63,6 +65,7 @@ class RazerDevice(DBusService):
         self.additional_interfaces = []
         if additional_interfaces is not None:
             self.additional_interfaces.extend(additional_interfaces)
+        self._battery_manager = None
 
         self.config = config
         self.persistence = persistence
@@ -232,7 +235,10 @@ class RazerDevice(DBusService):
                 self.add_dbus_method(m[0], m[1], m[2], in_signature=m[3], out_signature=m[4])
 
         for i in self.ZONES:
-            if 'set_' + i + '_static' in self.METHODS or 'set_' + i + '_static_naga_hex_v2' in self.METHODS or 'set_' + i + '_active' in self.METHODS:
+            if 'set_' + i + '_static_classic' in self.METHODS \
+                    or 'set_' + i + '_static' in self.METHODS \
+                    or 'set_' + i + '_active' in self.METHODS \
+                    or 'set_' + i + '_on' in self.METHODS:
                 self.zone[i]["present"] = True
                 for m in effect_methods[i]:
                     self.logger.debug("Adding {}.{} method to DBus".format(m[0], m[1]))
@@ -310,12 +316,21 @@ class RazerDevice(DBusService):
                     except (KeyError, configparser.NoOptionError):
                         self.logger.info("Failed to get " + i + " wave direction from persistence storage, using default.")
 
+        # Initialize battery manager if the device has support
+        if 'get_battery' in self.METHODS:
+            self._init_battery_manager()
+
         self.restore_dpi_poll_rate()
         self.restore_brightness()
 
         if self.config.getboolean('Startup', "restore_persistence") is True:
             self.restore_effect()
 
+            # Some devices need setting a second time after encountering Razer Synapse on Windows
+            if self.config.getboolean('Startup', "persistence_dual_boot_quirk") is True:
+                self.logger.debug("Restoring effect persistence again (dual boot quirk)")
+                self.restore_effect()
+
     def send_effect_event(self, effect_name, *args):
         """
         Send effect event
@@ -346,10 +361,23 @@ class RazerDevice(DBusService):
         """
         dpi_func = getattr(self, "setDPI", None)
         if dpi_func is not None:
+            # Constrain value in case the max has changed, e.g. wired/wireless might different maximums
+            if self.dpi[0] > self.DPI_MAX:
+                self.logger.warning("Constraining DPI X to maximum of " + str(self.DPI_MAX) + " because stored value " + str(self.dpi[0]) + " is larger.")
+                self.dpi[0] = self.DPI_MAX
+            if self.dpi[1] > self.DPI_MAX:
+                self.logger.warning("Constraining DPI Y to maximum of " + str(self.DPI_MAX) + " because stored value " + str(self.dpi[1]) + " is larger.")
+                self.dpi[1] = self.DPI_MAX
+
             dpi_func(self.dpi[0], self.dpi[1])
 
         poll_rate_func = getattr(self, "setPollRate", None)
         if poll_rate_func is not None:
+            # Constrain value in case the available values have changed, e.g. wired/wireless might different values available
+            if self.poll_rate not in self.POLL_RATES:
+                self.logger.warning("Constraining poll rate because stored value " + str(self.poll_rate) + " is not available.")
+                self.poll_rate = min(self.POLL_RATES, key=lambda x: abs(x - self.poll_rate))
+
             poll_rate_func(self.poll_rate)
 
     def restore_brightness(self):
@@ -444,6 +472,8 @@ class RazerDevice(DBusService):
                             effect_func(speed)
                         elif effect == 'wave':
                             effect_func(wave_dir)
+                        elif effect == 'wheel':
+                            effect_func(wave_dir)
                         elif effect == 'rippleRandomColour':
                             # do nothing. this is handled in the ripple manager.
                             pass
@@ -998,6 +1028,48 @@ class RazerDevice(DBusService):
 
             mode_file.write(bytes([mode_id, param]))
 
+    def _set_custom_effect(self):
+        """
+        Set the device to use custom LED matrix
+        """
+        # self.logger.debug("DBus call _set_custom_effect")
+
+        driver_path = self.get_driver_path('matrix_effect_custom')
+
+        payload = b'1'
+
+        with open(driver_path, 'wb') as driver_file:
+            driver_file.write(payload)
+
+    def _set_key_row(self, payload):
+        """
+        Set the RGB matrix on the device
+
+        Byte array like
+        [1, 255, 255, 00, 255, 255, 00, 255, 255, 00, 255, 255, 00, 255, 255, 00, 255, 255, 00, 255, 255, 00, 255, 255, 00,
+            255, 255, 00, 255, 255, 00, 255, 255, 00, 255, 255, 00, 255, 255, 00, 255, 255, 00, 255, 00, 00]
+
+        First byte is row, on firefly its always 1, on keyboard its 0-5
+        Then its 3byte groups of RGB
+        :param payload: Binary payload
+        :type payload: bytes
+        """
+        # self.logger.debug("DBus call set_key_row")
+
+        driver_path = self.get_driver_path('matrix_custom_frame')
+
+        with open(driver_path, 'wb') as driver_file:
+            driver_file.write(payload)
+
+    def _init_battery_manager(self):
+        """
+        Initializes the BatteryManager using the provided name
+        """
+        self._battery_manager = _BatteryManager(self, self._device_number, self.getDeviceName())  # pylint: disable=no-member
+        self._battery_manager.active = self.config.getboolean('Startup', 'battery_notifier', fallback=False)
+        self._battery_manager.frequency = self.config.getint('Startup', 'battery_notifier_freq', fallback=10 * 60)
+        self._battery_manager.percent = self.config.getint('Startup', 'battery_notifier_percent', fallback=33)
+
     def get_vid_pid(self):
         """
         Get the usb VID PID
@@ -1086,6 +1158,9 @@ class RazerDevice(DBusService):
         # Clear observer list
         self._observer_list.clear()
 
+        if self._battery_manager:
+            self._battery_manager.close()
+
     def close(self):
         """
         Close any resources opened by subclasses
